
long preheatDelay =                  90000;    // tube pre-heating delay
long rectifierDelay =                    5;    // rectifier pre-start delay
long inrushDelay =                    3000;    // inrush power delay
long overlapDelay =                    500;    // length of time between power on and inrush off


const int overvoltAlarm =                2;
const int standbyLED =                   4;
const int protectionLED =                6;
const int rectifierRelay =               7;
const int powerRelay =                   8;
const int inrushRelay =                  9;
const int preheatRelay =                10;
const int ACFailureAlarm =              11;
const int powerSwitch =                 12;
const int highVoltageLED =              14;


int powerButtonState                     ;
int lastPowerButtonState =           HIGH;

long lastDebounceTime =                 0;
long debounceDelay =                   50;          //switch debounce delay
long previousMillis =                   0;
unsigned long currentMillis =           0;
long overlapValue =                     0;
long inrushStartTime;
long preheatStartTime;
long rectifierStartTime;
long powerStartTime;
int preheatOn =                         0;
int rectifierOn =                       0;
int inrushOn =                          0;
int powerOn =                           0;
int overlapOn =                         0;
int indicatorStatus =                   0;
int ampMonitoring =                     0;



void setup() {
  pinMode   (overvoltAlarm, INPUT_PULLUP);
  pinMode            (standbyLED, OUTPUT);
  digitalWrite         (standbyLED, HIGH);             // turn off the LED
  pinMode         (protectionLED, OUTPUT);
  digitalWrite      (protectionLED, HIGH);             // turn off the LED
  pinMode        (rectifierRelay, OUTPUT);
  pinMode            (powerRelay, OUTPUT);
  pinMode           (inrushRelay, OUTPUT);
  pinMode          (preheatRelay, OUTPUT);
  pinMode     (powerSwitch, INPUT_PULLUP);
  pinMode  (ACFailureAlarm, INPUT_PULLUP);
  pinMode        (highVoltageLED, OUTPUT);
  digitalWrite     (highVoltageLED, HIGH);            // turn off the LED



  Serial.begin                     (9600);             // start serial communication for debugging
}

void loop() {
  checkPowerButton();
  preheat();
  startRectifier();
  inrush();
  power();
  overlap();
  checkOverVoltage();
  checkACFailure();
  checkStatusLED();
}

void checkOverVoltage() {
  if (ampMonitoring == 1) {
    int overvoltAlarmState = digitalRead (overvoltAlarm);
    if (overvoltAlarmState == LOW) {
      powerOff();
      indicatorStatus = 2;                                        // blink the status LED
      Serial.println ("Over voltage Shut Down ");
    }
  }
}

void checkACFailure() {
  if (ampMonitoring == 1) {
    int ACFailure = digitalRead(ACFailureAlarm);
    if (ACFailure == LOW) {
      powerOff();
      indicatorStatus = 3;                                        // blink the status LED
      Serial.println ("Power Loss Shut Down ");
    }
  }
}

void checkStatusLED() {
  if (indicatorStatus == 1) {           // normal operation, LED on
    digitalWrite (protectionLED, LOW);     
  }
  if (indicatorStatus == 2) {           // Over-Voltage blink pattern
    digitalWrite (protectionLED, LOW);
    delay(200);
    digitalWrite (protectionLED, HIGH);
    delay(1800);
  }
  if (indicatorStatus == 3) {          // AC Failure bliknk pattern
    digitalWrite (protectionLED, LOW);
    delay(100);
    digitalWrite (protectionLED, HIGH);
    delay(100);
    digitalWrite (protectionLED, LOW);
    delay(100);
    digitalWrite (protectionLED, HIGH);
    delay(1700);
  }
  if ( indicatorStatus == 0) {         // LED Off
    digitalWrite (protectionLED, HIGH);
  }
}

